-- Init
local frame = CreateFrame("FRAME", "TurnInEverything_Frame");
frame:RegisterEvent("GOSSIP_SHOW");
frame:RegisterEvent("QUEST_GREETING");
frame:RegisterEvent("QUEST_PROGRESS");
frame:RegisterEvent("QUEST_COMPLETE");
frame:RegisterEvent("MODIFIER_STATE_CHANGED");
-- Saved globals
tie_PermDisable = false;
tie_DEBUG = false;
-- Locals
local tie_TempDisable = false;
local tie_SkipNumGossips = 0;
local tie_LastGossipTime = 0;

-- Event handler
local function tie_EventHandler(self, event)
	if (tie_DEBUG) then
		print('tie_eventHandler:', event);
		print('tie_SkipNumGossips:', tie_SkipNumGossips);
		print('tie_LastGossipTime:', tie_LastGossipTime);
		print('tie_PermDisable', tie_PermDisable);
		print('tie_TempDisable', tie_TempDisable);
	end

	-- Track modifier changes
	if (event == "MODIFIER_STATE_CHANGED") then
		tie_ModiferChange();
		return;
	
	-- Track quest progress
	elseif (event == "QUEST_PROGRESS" or event == "QUEST_COMPLETE") then
		tie_QuestProgress(event);
		return;

	-- Track new gossip/greeting frames
	elseif (event == "GOSSIP_SHOW" or event == "QUEST_GREETING") then
		-- Do not process if disabled
		if (tie_isDisabled()) then
			return;
		end
		
		
		-- Do not process gossip/greeting if tie_SkipNumGossips > 0
		if (tie_SkipNumGossips > 0) then
			-- Ignore tie_SkipNumGossips if it's been more than a few of seconds (in case GOSSIP_SHOW only fired once)
			if (tie_LastGossipTime + 3 < GetTime()) then
				if (tie_DEBUG) then print('tie_EventHandler: Ignoring tie_SkipNextGossip due to age'); end
				tie_SkipNumGossips = 0;
			-- Otherwise decrement tie_SkipNumGossips and skip this round
			else
				if (tie_DEBUG) then print('tie_EventHandler: Exiting on tie_SkipNextGossip'); end
				tie_SkipNumGossips = tie_SkipNumGossips - 1;
				return;
			end
		end
		
		-- Record the processing time
		tie_LastGossipTime = GetTime();
		
		-- Process each gossip/greeting option
		if (event == "GOSSIP_SHOW") then
			tie_GossipShow();
		else
			tie_QuestGreeting();
		end
		return;
	end
end
frame:SetScript('OnEvent', tie_EventHandler);

-- Slash commands
SLASH_TURNINEVERYTHING1, SLASH_TURNINEVERYTHING2 = '/tie', '/turnineverything';
SlashCmdList["TURNINEVERYTHING"] = function(msg, editbox)
	outmsg = "Unrecognized command";
	if (msg == 'disable') then
		tie_PermDisable = true;
		outmsg = "Disabled";
	elseif (msg == 'enable') then
		tie_PermDisable = false;
		outmsg = "Enabled";
	elseif (msg == 'debug') then
		if (tie_DEBUG) then
			tie_DEBUG = false;
			outmsg = "Debug Disabled";
		else
			tie_DEBUG = true;
			outmsg = "Debug Enabled";
		end
	end
	print("TurnInEverything: " .. outmsg);
end

-- Check each active quest
function tie_QuestGreeting()
	-- If there are any active quests
	numQ = GetNumActiveQuests();
	if (numQ ~= nil) then
	
		-- Check each one
		for i = 1, numQ do
			if (tie_DEBUG) then print('tie_QuestGreeting: Processing quest:', i); end
			SelectActiveQuest(i);
		
			-- Skip the next QUEST_GREEETING event
			tie_SkipNumGossips = 1;
		end
	end
end

-- Check each active quest
function tie_GossipShow()
	-- Find all active quests
	local q = { GetGossipActiveQuests() };
	local i = 1;
	local qNum = 1;

	-- Loop through the quest details
	-- We need this i+3 complication to avoid the nils in q
	while(q[i]) do
		if (tie_DEBUG) then print('tie_GossipShow: Processing quest:', qNum); end
		
		-- Select the quest and let it throw QUEST_PROGRESS
		SelectGossipActiveQuest(qNum);
		i = i + 3;
		qNum = qNum + 1;

		-- Skip two GOSSIP_SHOW events, as they are often doubled
		tie_SkipNumGossips = 2;
	end
end

-- If this quest is completable, click continue and spew out text until we get to the completion screen
function tie_QuestProgress(event)
	-- If we're done with the progress text
	if (event == "QUEST_COMPLETE") then
		-- Display the reward text
		DEFAULT_CHAT_FRAME:AddMessage(GetTitleText() .. ' Reward: ' .. GetRewardText(), 1, 0.98, 0.8);
	
		-- Do not accept the reward if disabled
		if (tie_isDisabled()) then
			return;
		end
		
		-- If there's only one reward, grab it
		if (GetNumQuestChoices() < 2) then
			GetQuestReward(1);
		end

	-- If there's still progress text left
	else
		-- Display the progress text
		DEFAULT_CHAT_FRAME:AddMessage(GetTitleText() .. ' Progress: ' .. GetProgressText(), 1, 0.98, 0.8);

		-- Do not complete the quest if disabled
		if (tie_isDisabled()) then
			return;
		end

		-- Bail if the quest is not completable
		if(not IsQuestCompletable()) then
			DeclineQuest();
			return;
		end

		-- Otherwise click continue continue
		CompleteQuest();
	end
end

-- Temporarily disable processing if the hotkey is pressed
-- Currently hard-coded to shift, but abstracted to a function should I ever to to writing a UI to allow user configuration
function tie_ModiferChange()
	if (IsShiftKeyDown()) then
		tie_TempDisable = true;
	else
		tie_TempDisable = false;
	end
end

-- Return true if TIE is disabled for any reason
function tie_isDisabled()
	if (tie_TempDisable or tie_PermDisable) then
		return true;
	end
	return false;
end